@@ -273,6 +273,12 @@ class CanvasMainWindow(QMainWindow):
         self.dock_help = canvas_tool_dock.help
         self.dock_help.setMaximumHeight(150)
         self.dock_help.document().setDefaultStyleSheet("h3, a {color: orange;}")
+        default_help = "Select a widget to show its description." \
+                       "<br><br>" \
+                       "See <a href='orange://examples'>workflow examples</a>, " \
+                       "<a href='orange://tutorials'>YouTube tutorials</a>, " \
+                       "or open the <a href='orange://welcome'>welcome screen</a>."
+        self.dock_help.setDefaultText(default_help)
 
         self.dock_help_action = canvas_tool_dock.toogleQuickHelpAction()
         self.dock_help_action.setText(self.tr("Show Help"))
@@ -460,7 +466,7 @@ class CanvasMainWindow(QMainWindow):
                     )
 
         self.tutorials_action = \
-            QAction(self.tr("Tutorials"), self,
+            QAction(self.tr("YouTube Tutorials"), self,
                     objectName="tutorials-action",
                     toolTip=self.tr("View YouTube tutorials."),
                     triggered=self.tutorials,
@@ -468,7 +474,7 @@ class CanvasMainWindow(QMainWindow):
                     )
 
         self.examples_action = \
-            QAction(self.tr("Examples"), self,
+            QAction(self.tr("Workflow Examples"), self,
                     objectName="tutorial-action",
                     toolTip=self.tr("Browse example workflows."),
                     triggered=self.tutorial_scheme,
@@ -688,7 +694,7 @@ class CanvasMainWindow(QMainWindow):
         window_groups = self.scheme_widget.findChild(
             QAction, "window-groups-action"
         )
-        if isinstance(window_groups, QAction):
+        if window_groups is not None:
             self.view_menu.addAction(window_groups)
 
         self.view_menu.addSeparator()
@@ -704,6 +710,12 @@ class CanvasMainWindow(QMainWindow):
         self.view_menu.addSeparator()
 
         self.view_menu.addAction(self.toogle_margins_action)
+        raise_widgets_action = self.scheme_widget.findChild(
+            QAction, "bring-widgets-to-front-action"
+        )
+        if raise_widgets_action is not None:
+            self.view_menu.addAction(raise_widgets_action)
+
         self.view_menu.addAction(self.float_widgets_on_top_action)
         menu_bar.addMenu(self.view_menu)
 
@@ -1099,6 +1111,9 @@ class CanvasMainWindow(QMainWindow):
             scheme_doc_widget.setPath(filename)
 
             self.add_recent_scheme(new_scheme.title, filename)
+            if not self.freeze_action.isChecked():
+                # activate the default window group.
+                scheme_doc_widget.activateDefaultWindowGroup()
 
     def load_scheme_xml(self, xml):
         new_scheme = widgetsscheme.WidgetsScheme(parent=self)
@@ -1316,6 +1331,7 @@ class CanvasMainWindow(QMainWindow):
         # existing scheme file if `scheme.save_to` raises an error.
         buffer = BytesIO()
         try:
+            scheme.set_runtime_env("basedir", dirname)
             scheme.save_to(buffer, pretty=True, pickle_fallback=True)
         except Exception:
             log.error("Error saving %r to %r", scheme, filename, exc_info=True)
@@ -1331,7 +1347,6 @@ class CanvasMainWindow(QMainWindow):
         try:
             with open(filename, "wb") as f:
                 f.write(buffer.getvalue())
-            scheme.set_runtime_env("basedir", dirname)
             return True
         except (IOError, OSError) as ex:
             log.error("%s saving '%s'", type(ex).__name__, filename,
@@ -1522,12 +1537,13 @@ class CanvasMainWindow(QMainWindow):
                     )
 
         examples_action = \
-            QAction(self.examples_action.text(), dialog,
-                    icon=self.examples_action.icon(),
-                    toolTip=self.examples_action.toolTip(),
-                    whatsThis=self.examples_action.whatsThis(),
-                    triggered=open_examples,
+            QAction(self.tr("Examples"), self,
+                    icon=canvas_icons("Examples.svg"),
+                    toolTip=self.tr("Browse example workflows."),
+                    objectName="tutorial-action",
+                    triggered=open_examples
                     )
+
         tutorials_action = \
             QAction(self.tr("Tutorials"), self,
                     objectName="tutorials-action",
@@ -1913,16 +1929,22 @@ class CanvasMainWindow(QMainWindow):
             if url.scheme() == "help" and url.authority() == "search":
                 try:
                     url = self.help.search(url)
+                    self.show_help(url)
                 except KeyError:
-                    url = None
                     log.info("No help topic found for %r", url)
-
-            if url:
-                self.show_help(url)
-            else:
-                message_information(
-                    self.tr("There is no documentation for this widget yet."),
-                    parent=self)
+                    message_information(
+                        self.tr("There is no documentation for this widget yet."),
+                        parent=self)
+            elif url.scheme() == "orange":
+                target = url.host()
+                if target == "examples":
+                    self.tutorial_scheme()
+                elif target == "tutorials":
+                    self.tutorials()
+                elif target == "welcome":
+                    self.welcome_dialog()
+                else:
+                    log.error("No target found for %r", url)
 
             return True
 
