@@ -27,6 +27,7 @@ class OWMergeData(widget.OWWidget):
     description = "Merge datasets based on the values of selected features."
     icon = "icons/MergeData.svg"
     priority = 1110
+    keywords = ["join"]
 
     class Inputs:
         data = Input("Data", Orange.data.Table, default=True, replaces=["Data A"])
@@ -44,6 +45,7 @@ class OWMergeData(widget.OWWidget):
     attr_combine_data = settings.Setting('', schema_only=True)
     attr_combine_extra = settings.Setting('', schema_only=True)
     merging = settings.Setting(0)
+    auto_apply = settings.Setting(True)
 
     want_main_area = False
     resizing_enabled = False
@@ -111,6 +113,9 @@ class OWMergeData(widget.OWWidget):
                    self.model_unique_with_id, self.extra_model_unique_with_id)
         self.set_merging()
 
+        gui.auto_commit(self.controlArea, self, "auto_apply", "&Apply",
+                        box=False)
+
     def set_merging(self):
         # pylint: disable=invalid-sequence-index
         # all boxes should be hidden before one is shown, otherwise widget's
