@@ -11,7 +11,7 @@ from xml.sax.saxutils import escape
 
 from AnyQt.QtCore import (
     Qt, QObject, QAbstractListModel, QAbstractTableModel, QModelIndex,
-    QItemSelectionModel, QT_VERSION
+    QItemSelectionModel, QMimeData, QT_VERSION
 )
 from AnyQt.QtCore import pyqtSignal as Signal
 from AnyQt.QtGui import QColor
@@ -540,8 +540,6 @@ class PyListModel(QAbstractListModel):
         if role == Qt.DisplayRole:
             return str(section)
 
-
-    # noinspection PyMethodOverriding
     def rowCount(self, parent=QModelIndex()):
         return 0 if parent.isValid() else len(self._list)
 
@@ -572,7 +570,8 @@ class PyListModel(QAbstractListModel):
     def setData(self, index, value, role=Qt.EditRole):
         if role == Qt.EditRole:
             if self._is_index_valid(index):
-                self[index.row()] = value  # Will emit proper dataChanged signal
+                self._list[index.row()] = value
+                self.dataChanged.emit(index, index)
                 return True
         elif self._is_index_valid(index):
             self._other_data[index.row()][role] = value
@@ -582,11 +581,14 @@ class PyListModel(QAbstractListModel):
 
     def setItemData(self, index, data):
         data = dict(data)
+        if not data:
+            return True  # pragma: no cover
+
         with signal_blocking(self):
             for role, value in data.items():
                 if role == Qt.EditRole and \
                         self._is_index_valid(index):
-                    self[index.row()] = value
+                    self._list[index.row()] = value
                 elif self._is_index_valid(index):
                     self._other_data[index.row()][role] = value
 
@@ -599,8 +601,6 @@ class PyListModel(QAbstractListModel):
         else:
             return self._flags | Qt.ItemIsDropEnabled
 
-
-    # noinspection PyMethodOverriding
     def insertRows(self, row, count, parent=QModelIndex()):
         """ Insert ``count`` rows at ``row``, the list fill be filled
         with ``None``
@@ -611,8 +611,6 @@ class PyListModel(QAbstractListModel):
         else:
             return False
 
-
-    # noinspection PyMethodOverriding
     def removeRows(self, row, count, parent=QModelIndex()):
         """Remove ``count`` rows starting at ``row``
         """
@@ -622,6 +620,28 @@ class PyListModel(QAbstractListModel):
         else:
             return False
 
+    def moveRows(self, sourceParent, sourceRow, count,
+                 destinationParent, destinationChild):
+        # type: (QModelIndex, int, int, QModelIndex, int) -> bool
+        """
+        Move `count` rows starting at `sourceRow` to `destinationChild`.
+
+        Reimplemented from QAbstractItemModel.moveRows
+        """
+        if not self.beginMoveRows(sourceParent, sourceRow, sourceRow + count - 1,
+                                  destinationParent, destinationChild):
+            return False
+        take_slice = slice(sourceRow, sourceRow + count)
+        insert_at = destinationChild
+        if insert_at > sourceRow:
+            insert_at -= count
+        items, other = self._list[take_slice], self._other_data[take_slice]
+        del self._list[take_slice], self._other_data[take_slice]
+        self._list[insert_at:insert_at] = items
+        self._other_data[insert_at: insert_at] = other
+        self.endMoveRows()
+        return True
+
     def extend(self, iterable):
         list_ = list(iterable)
         self.beginInsertRows(QModelIndex(),
@@ -752,15 +772,23 @@ class PyListModel(QAbstractListModel):
         return self._supportedDropActions
 
     def mimeTypes(self):
-        return [self.MIME_TYPE] + list(QAbstractListModel.mimeTypes(self))
+        return [self.MIME_TYPE]
 
     def mimeData(self, indexlist):
         if len(indexlist) <= 0:
             return None
 
+        def itemData(row):
+            # type: (int) -> Dict[int, Any]
+            if row < len(self._other_data):
+                return {key: val for key, val in self._other_data[row].items()
+                        if isinstance(key, int)}
+            else:
+                return {}  # pragma: no cover
+
         items = [self[i.row()] for i in indexlist]
-        itemdata = [self.itemData(i) for i in indexlist]
-        mime = QAbstractListModel.mimeData(self, indexlist)
+        itemdata = [itemData(i.row()) for i in indexlist]
+        mime = QMimeData()
         mime.setData(self.MIME_TYPE, b'see properties: _items, _itemdata')
         mime.setProperty('_items', items)
         mime.setProperty('_itemdata', itemdata)
@@ -768,19 +796,19 @@ class PyListModel(QAbstractListModel):
 
     def dropMimeData(self, mime, action, row, column, parent):
         if action == Qt.IgnoreAction:
-            return True
+            return True  # pragma: no cover
 
         if not mime.hasFormat(self.MIME_TYPE):
-            return False
+            return False  # pragma: no cover
 
         items = mime.property('_items')
         itemdata = mime.property('_itemdata')
 
         if not items:
-            return False
+            return False  # pragma: no cover
 
         if row == -1:
-            row = len(self)
+            row = len(self)  # pragma: no cover
 
         self[row:row] = items
         for i, data in enumerate(itemdata):
@@ -1006,6 +1034,27 @@ class DomainModel(VariableListModel):
     def sort(self, *args, **kwargs):
         return super().sort(*args, **kwargs)
 
+    def setData(self, index, value, role=Qt.EditRole):
+        # reimplemented
+        if role == Qt.EditRole:
+            return False
+        else:
+            return super().setData(index, value, role)
+
+    def setItemData(self, index, data):
+        # reimplemented
+        if Qt.EditRole in data:
+            return False
+        else:
+            return super().setItemData(index, data)
+
+    def insertRows(self, row, count, parent=QModelIndex()):
+        # reimplemented
+        return False
+
+    def removeRows(self, row, count, parent=QModelIndex()):
+        # reimplemented
+        return False
 
 _html_replace = [("<", "&lt;"), (">", "&gt;")]
 
