@@ -9,10 +9,11 @@ from operator import attrgetter
 from AnyQt.QtWidgets import (
     QWidget, QDialog, QVBoxLayout, QSizePolicy, QApplication, QStyle,
     QShortcut, QSplitter, QSplitterHandle, QPushButton, QStatusBar,
-    QProgressBar, QAction
+    QProgressBar, QAction, QWIDGETSIZE_MAX
 )
 from AnyQt.QtCore import (
-    Qt, QByteArray, QSettings, QUrl, pyqtSignal as Signal
+    Qt, QRect, QMargins, QByteArray, QDataStream, QBuffer, QSettings,
+    QUrl, pyqtSignal as Signal
 )
 from AnyQt.QtGui import QIcon, QKeySequence, QDesktopServices
 
@@ -235,12 +236,13 @@ class OWWidget(QDialog, OWComponent, Report, ProgressBarMixin,
             self.controlArea.setFocus(Qt.ActiveWindowFocusReason)
 
         if self.__splitter is not None:
-            self.__splitter.controlAreaVisibilityChanged.connect(
-                self.storeControlAreaVisibility)
+            self.__splitter.handleClicked.connect(
+                self.__toggleControlArea
+            )
             sc = QShortcut(
                 QKeySequence(Qt.ControlModifier | Qt.ShiftModifier | Qt.Key_D),
-                self)
-            sc.activated.connect(self.__splitter.flip)
+                self, autoRepeat=False)
+            sc.activated.connect(self.__toggleControlArea)
         return self
 
     # pylint: disable=super-init-not-called
@@ -268,8 +270,7 @@ class OWWidget(QDialog, OWComponent, Report, ProgressBarMixin,
                 else Qt.Dialog | Qt.MSWindowsFixedSizeDialogHint)
 
     class _Splitter(QSplitter):
-        controlAreaVisibilityChanged = Signal(int)
-
+        handleClicked = Signal()
         def _adjusted_size(self, size_method):
             size = size_method(super())()
             height = max((size_method(self.widget(i))().height()
@@ -289,26 +290,11 @@ class OWWidget(QDialog, OWComponent, Report, ProgressBarMixin,
             return self._Handle(
                 self.orientation(), self, cursor=Qt.PointingHandCursor)
 
-        def flip(self):
-            if self.count() == 1:  # Prevent hiding control area by shortcut
-                return
-            self.setControlAreaVisible(not self.controlAreaVisible())
-
-        def setControlAreaVisible(self, visible):
-            if self.controlAreaVisible() == visible:
-                return
-            self.setSizes([int(visible), 100000])
-            self.controlAreaVisibilityChanged.emit(visible)
-            self.updateGeometry()
-
-        def controlAreaVisible(self):
-            return bool(self.sizes()[0])
-
         class _Handle(QSplitterHandle):
             def mouseReleaseEvent(self, event):
                 """Resize on left button"""
                 if event.button() == Qt.LeftButton:
-                    self.splitter().flip()
+                    self.splitter().handleClicked.emit()
                 super().mouseReleaseEvent(event)
 
             def mouseMoveEvent(self, event):
@@ -397,8 +383,10 @@ class OWWidget(QDialog, OWComponent, Report, ProgressBarMixin,
             c.layout().addWidget(sb)
 
             help = self.__help_action
+            icon = QIcon(gui.resource_filename("icons/help.svg"))
+            icon.addFile(gui.resource_filename("icons/help-hover.svg"), mode=QIcon.Active)
             help_button = SimpleButton(
-                icon=QIcon(gui.resource_filename("icons/help.svg")),
+                icon=icon,
                 toolTip="Show widget help", visible=help.isVisible(),
             )
             @help.changed.connect
@@ -409,15 +397,19 @@ class OWWidget(QDialog, OWComponent, Report, ProgressBarMixin,
             sb.addWidget(help_button)
 
             if self.graph_name is not None:
+                icon = QIcon(gui.resource_filename("icons/chart.svg"))
+                icon.addFile(gui.resource_filename("icons/chart-hover.svg"), mode=QIcon.Active)
                 b = SimpleButton(
-                    icon=QIcon(gui.resource_filename("icons/chart.svg")),
+                    icon=icon,
                     toolTip="Save Image",
                 )
                 b.clicked.connect(self.save_graph)
                 sb.addWidget(b)
             if hasattr(self, "send_report"):
+                icon = QIcon(gui.resource_filename("icons/report.svg"))
+                icon.addFile(gui.resource_filename("icons/report-hover.svg"), mode=QIcon.Active)
                 b = SimpleButton(
-                    icon=QIcon(gui.resource_filename("icons/report.svg")),
+                    icon=icon,
                     toolTip="Report"
                 )
                 b.clicked.connect(self.show_report)
@@ -452,6 +444,71 @@ class OWWidget(QDialog, OWComponent, Report, ProgressBarMixin,
             margins.setBottom(sb.sizeHint().height())
             self.setContentsMargins(margins)
 
+    def __toggleControlArea(self):
+        if self.__splitter is None or self.__splitter.count() < 2:
+            return
+        self.__setControlAreaVisible(not self.__splitter.sizes()[0])
+
+    def __setControlAreaVisible(self, visible):
+        # type: (bool) -> None
+        if self.__splitter is None or self.__splitter.count() < 2:
+            return
+        self.controlAreaVisible = visible
+        splitter = self.__splitter  # type: QSplitter
+        w = splitter.widget(0)
+        # Set minimum width to 1 (overrides minimumSizeHint) when control area
+        # is not visible to allow the main area to shrink further. Reset the
+        # minimum width with a 0 if control area is visible.
+        w.setMinimumWidth(int(not visible))
+
+        sizes = splitter.sizes()
+        current_size = sizes[0]
+        if bool(current_size) == visible:
+            return
+
+        current_width = w.width()
+        geom = self.geometry()
+        frame = self.frameGeometry()
+        framemargins = QMargins(
+            frame.left() - geom.left(),
+            frame.top() - geom.top(),
+            frame.right() - geom.right(),
+            frame.bottom() - geom.bottom()
+        )
+        splitter.setSizes([int(visible), QWIDGETSIZE_MAX])
+        if not self.isWindow() or \
+                self.windowState() not in [Qt.WindowNoState, Qt.WindowActive]:
+            # not a window or not in state where we can move move/resize
+            return
+
+        # force immediate resize recalculation
+        splitter.refresh()
+        self.layout().invalidate()
+        self.layout().activate()
+
+        if visible:
+            # move left and expand by the exposing widget's width
+            diffx = -w.width()
+            diffw = w.width()
+        else:
+            # move right and shrink by the collapsing width
+            diffx = current_width
+            diffw = -current_width
+        newgeom = QRect(
+            geom.x() + diffx, geom.y(), geom.width() + diffw, geom.height()
+        )
+        # bound/move by available geometry
+        bounds = QApplication.desktop().availableGeometry(self)
+        bounds = bounds.adjusted(
+            framemargins.left(), framemargins.top(),
+            -framemargins.right(), -framemargins.bottom()
+        )
+        newsize = newgeom.size().boundedTo(bounds.size())
+        newgeom = QRect(newgeom.topLeft(), newsize)
+        newgeom.moveLeft(max(newgeom.left(), bounds.left()))
+        newgeom.moveRight(min(newgeom.right(), bounds.right()))
+        self.setGeometry(newgeom)
+
     def save_graph(self):
         """Save the graph with the name given in class attribute `graph_name`.
 
@@ -470,11 +527,8 @@ class OWWidget(QDialog, OWComponent, Report, ProgressBarMixin,
                 return
             ClipboardFormat.write_image(None, graph_obj)
 
-    def storeControlAreaVisibility(self, visible):
-        self.controlAreaVisible = visible
-
-    def __restoreWidgetGeometry(self):
-
+    def __restoreWidgetGeometry(self, geometry):
+        # type: (bytes) -> bool
         def _fullscreen_to_maximized(geometry):
             """Don't restore windows into full screen mode because it loses
             decorations and can't be de-fullscreened at least on some platforms.
@@ -487,31 +541,29 @@ class OWWidget(QDialog, OWComponent, Report, ProgressBarMixin,
             return w.saveGeometry()
 
         restored = False
-        if self.save_position:
-            geometry = self.savedWidgetGeometry
-            if geometry is not None:
-                geometry = _fullscreen_to_maximized(geometry)
-                restored = self.restoreGeometry(geometry)
+        if geometry:
+            geometry = _fullscreen_to_maximized(geometry)
+            restored = self.restoreGeometry(geometry)
 
-            if restored and not self.windowState() & \
-                    (Qt.WindowMaximized | Qt.WindowFullScreen):
-                space = QApplication.desktop().availableGeometry(self)
-                frame, geometry = self.frameGeometry(), self.geometry()
+        if restored and not self.windowState() & \
+                (Qt.WindowMaximized | Qt.WindowFullScreen):
+            space = QApplication.desktop().availableGeometry(self)
+            frame, geometry = self.frameGeometry(), self.geometry()
 
-                #Fix the widget size to fit inside the available space
-                width = space.width() - (frame.width() - geometry.width())
-                width = min(width, geometry.width())
-                height = space.height() - (frame.height() - geometry.height())
-                height = min(height, geometry.height())
-                self.resize(width, height)
+            # Fix the widget size to fit inside the available space
+            width = space.width() - (frame.width() - geometry.width())
+            width = min(width, geometry.width())
+            height = space.height() - (frame.height() - geometry.height())
+            height = min(height, geometry.height())
+            self.resize(width, height)
 
-                # Move the widget to the center of available space if it is
-                # currently outside it
-                if not space.contains(self.frameGeometry()):
-                    x = max(0, space.width() / 2 - width / 2)
-                    y = max(0, space.height() / 2 - height / 2)
+            # Move the widget to the center of available space if it is
+            # currently outside it
+            if not space.contains(self.frameGeometry()):
+                x = max(0, space.width() / 2 - width / 2)
+                y = max(0, space.height() / 2 - height / 2)
 
-                    self.move(x, y)
+                self.move(x, y)
 
         # Mark as explicitly moved/resized if not already. QDialog would
         # otherwise adjust position/size on subsequent hide/show
@@ -573,8 +625,9 @@ class OWWidget(QDialog, OWComponent, Report, ProgressBarMixin,
         if self.save_position and not self.__was_restored:
             # Restore saved geometry on (first) show
             if self.__splitter is not None:
-                self.__splitter.setControlAreaVisible(self.controlAreaVisible)
-            self.__restoreWidgetGeometry()
+                self.__setControlAreaVisible(self.controlAreaVisible)
+            if self.savedWidgetGeometry is not None:
+                self.__restoreWidgetGeometry(bytes(self.savedWidgetGeometry))
             self.__was_restored = True
         self.__quicktipOnce()
 
@@ -770,6 +823,65 @@ class OWWidget(QDialog, OWComponent, Report, ProgressBarMixin,
         """
         pass
 
+    def saveGeometryAndLayoutState(self):
+        # type: () -> QByteArray
+        """
+        Save the current geometry and layout state of this widget and
+        child windows (if applicable).
+
+        Returns
+        -------
+        state : QByteArray
+            Saved state.
+        """
+        version = 0x1
+        have_spliter = 0
+        splitter_state = 0
+        if self.__splitter is not None:
+            have_spliter = 1
+            splitter_state = 1 if self.controlAreaVisible else 0
+        data = QByteArray()
+        stream = QDataStream(data, QBuffer.WriteOnly)
+        stream.writeUInt32(version)
+        stream.writeUInt16((have_spliter << 1) | splitter_state)
+        stream <<= self.saveGeometry()
+        return data
+
+    def restoreGeometryAndLayoutState(self, state):
+        # type: (QByteArray) -> bool
+        """
+        Restore the geometry and layout of this widget to a state previously
+        saved with :func:`saveGeometryAndLayoutState`.
+
+        Parameters
+        ----------
+        state : QByteArray
+            Saved state.
+
+        Returns
+        -------
+        success : bool
+            `True` if the state was successfully restored, `False` otherwise.
+        """
+        version = 0x1
+        stream = QDataStream(state, QBuffer.ReadOnly)
+        version_ = stream.readUInt32()
+        if stream.status() != QDataStream.Ok or version_ != version:
+            return False
+        splitter_state = stream.readUInt16()
+        has_spliter = splitter_state & 0x2
+        splitter_state = splitter_state & 0x1
+        if has_spliter and self.__splitter is not None:
+            self.__setControlAreaVisible(bool(splitter_state))
+        geometry = QByteArray()
+        stream >>= geometry
+        if stream.status() == QDataStream.Ok:
+            state = self.__restoreWidgetGeometry(bytes(geometry))
+            self.__was_restored = self.__was_restored or state
+            return state
+        else:
+            return False  # pragma: no cover
+
     def __showMessage(self, message):
         if self.__msgwidget is not None:
             self.__msgwidget.hide()
